<?php
/**
 * Register custom blocks
 *
 * @package    BoldBlocks
 * @author     Phi Phan <mrphipv@gmail.com>
 * @copyright  Copyright (c) 2022, Phi Phan
 */

namespace BoldBlocks;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'CustomBlocks' ) ) :
	/**
	 * Create/edit custom content blocks.
	 */
	class CustomBlocks extends CoreComponent {
		/**
		 * Store all custom blocks
		 *
		 * @var array
		 */
		protected $all_custom_blocks = [];

		/**
		 * Store all names of custom blocks
		 *
		 * @var array
		 */
		protected $custom_blocks = [];

		/**
		 * Store all names of repeater blocks
		 *
		 * @var array
		 */
		protected $repeater_blocks = [];

		/**
		 * Max items that can query
		 *
		 * @var integer
		 */
		protected $max_items = 100;

		/**
		 * Run main hooks
		 *
		 * @return void
		 */
		public function run() {
			// Register custom post type.
			add_action( 'init', [ $this, 'register_custom_post_type' ], 5 );

			// Custom admin columns.
			add_filter( 'manage_edit-boldblocks_block_columns', [ $this, 'add_boldblocks_block_custom_columns' ] );
			add_action( 'manage_boldblocks_block_posts_custom_column', [ $this, 'manage_boldblocks_block_columns' ], 10, 2 );
			add_action( 'admin_head', [ $this, 'add_custom_columns_style' ] );

			// Load custom blocks.
			add_action( 'init', [ $this, 'load_custom_blocks' ] );

			// Register scripts.
			add_action( 'init', [ $this, 'register_scripts' ] );

			// Allow creating patterns from custom blocks.
			add_filter( 'boldblocks_get_pattern_allowed_blocks', [ $this, 'allow_creating_patterns' ] );

			// Add meta fields to meta revisioning.
			add_filter( 'boldblocks_post_revision_meta_keys', [ $this, 'add_fields_to_revision_meta_keys' ] );

			// Add theme name to body class.
			add_filter( 'body_class', [ $this, 'add_body_class' ] );

			// Add block version to custom blocks.
			add_action( 'save_post_boldblocks_block', [ $this, 'save_version_to_block' ], 10, 3 );

			// Prevent users from deleting the core blocks.
			add_filter( 'user_has_cap', [ $this, 'prevent_core_blocks_deletion' ], 10, 3 );

			// Change the placeholder text.
			add_filter( 'enter_title_here', [ $this, 'enter_title_here' ] );

			// Register codemirror.
			add_action( 'admin_enqueue_scripts', [ $this, 'register_code_editor_scripts' ] );

			// Clear transient cache.
			add_action( 'save_post_boldblocks_block', [ $this, 'clear_transient_cache' ] );

			// Enqueue frontend scripts for carousel layout.
			add_filter( 'render_block', [ $this, 'enqueue_frontend_carousel_scripts' ], 10, 3 );
		}

		/**
		 * Register custom post type
		 *
		 * @return void
		 */
		public function register_custom_post_type() {
			// Custom block args.
			$labels = array(
				'name'               => _x( 'Custom Blocks', 'post type general name', 'content-blocks-builder' ),
				'singular_name'      => _x( 'Custom Block', 'post type singular name', 'content-blocks-builder' ),
				'menu_name'          => _x( 'Custom Blocks', 'admin menu', 'content-blocks-builder' ),
				'name_admin_bar'     => _x( 'Custom Blocks', 'add new on admin bar', 'content-blocks-builder' ),
				'add_new'            => _x( 'Add New Block', 'item', 'content-blocks-builder' ),
				'add_new_item'       => __( 'Add New Custom Block', 'content-blocks-builder' ),
				'new_item'           => __( 'New Custom Block', 'content-blocks-builder' ),
				'edit_item'          => __( 'Edit Custom Block', 'content-blocks-builder' ),
				'view_item'          => __( 'View Custom Block', 'content-blocks-builder' ),
				'all_items'          => __( 'All Blocks', 'content-blocks-builder' ),
				'search_items'       => __( 'Search Custom Blocks', 'content-blocks-builder' ),
				'parent_item_colon'  => __( 'Parent Custom Block:', 'content-blocks-builder' ),
				'not_found'          => __( 'No items found.', 'content-blocks-builder' ),
				'not_found_in_trash' => __( 'No items found in trash.', 'content-blocks-builder' ),
			);

			$args = array(
				'labels'              => $labels,
				'description'         => __( 'All custom blocks', 'content-blocks-builder' ),
				'public'              => true,
				'publicly_queryable'  => false,
				'exclude_from_search' => true,
				'show_ui'             => true,
				'show_in_menu'        => true,
				'show_in_nav_menus'   => false,
				'show_in_admin_bar'   => true,
				'show_in_rest'        => true,
				'rest_base'           => 'boldblocks-blocks',
				'query_var'           => false,
				'rewrite'             => false,
				'capability_type'     => 'post',
				'has_archive'         => false,
				'hierarchical'        => false,
				'menu_position'       => 80, // Right after Settings.
				'menu_icon'           => 'dashicons-block-default',
				'supports'            => array( 'title', 'editor', 'author', 'custom-fields', 'revisions', 'page-attributes' ),
				'can_export'          => true,
				'template_lock'       => 'insert',
			);

			// Register post type.
			register_post_type( 'boldblocks_block', $args );

			// Meta fields.
			register_meta(
				'post',
				'boldblocks_block_blocks',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '[]',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_block_dependent_blocks',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => array(
						'schema' => array(
							'items' => array(
								'type' => 'string',
							),
						),
					),
					'type'           => 'array',
					'default'        => [],
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_block_class',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_block_description',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_block_icon',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_block_supports',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => [
						'schema' => [
							'type'                 => 'object',
							'additionalProperties' => array(
								'type' => [ 'boolean' ], // can be ['boolean', 'object'...].
							),
						],
					],
					'type'           => 'object',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_template_lock',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => 'all',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_enable_variation_picker',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'boolean',
					'default'        => false,
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_enable_repeater',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'boolean',
					'default'        => false,
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_parent_layout_type',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => 'vstack',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_parent_block_icon',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_parent_block_supports',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => [
						'schema' => [
							'type'                 => 'object',
							'additionalProperties' => array(
								'type' => [ 'boolean' ], // can be ['boolean', 'object'...].
							),
						],
					],
					'type'           => 'object',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_parent_enable_variation_picker',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'boolean',
					'default'        => false,
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_disable_standalone',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'boolean',
					'default'        => false,
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_is_fixed_nested_item_count',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'boolean',
					'default'        => false,
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_nested_item_count',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'integer',
					'default'        => 1,
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_parent_block_name',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_parent_block_title',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_parent_block_description',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_parent_block_class',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_block_version',
				array(
					'object_subtype' => 'boldblocks_block',
					'show_in_rest'   => true,
					'type'           => 'string',
					'default'        => '',
					'single'         => true,
				)
			);
		}

		/**
		 * Load all custom blocks
		 *
		 * @return void
		 */
		public function load_custom_blocks() {
			// Load all custom blocks.
			$all_custom_blocks = $this->get_all_custom_blocks();

			// Build an array of custom block name.
			$this->custom_blocks = array_column( $all_custom_blocks, null, 'name' );

			// Build an array of repeater blocks.
			$this->repeater_blocks = array_filter(
				array_map(
					function ( $block_args ) {
						return isset( $block_args['parent'] ) ? $block_args['parent'] : false;
					},
					$all_custom_blocks
				)
			);

			// Make name as the key.
			$this->repeater_blocks = array_column( $this->repeater_blocks, null, 'name' );

			// Register all blocks.
			foreach ( array_merge( $this->custom_blocks, $this->repeater_blocks ) as $name => $args ) {
				// Build block attributes.
				$block_atts = [
					'api_version'   => 2,
					'supports'      => [
						'cbb'                  => true,
						'__experimentalLayout' => true,
					],
					'uses_context'  => [ 'postId', 'postType' ],
					'style'         => 'boldblocks-custom-blocks',
					'editor_script' => 'boldblocks-custom-blocks',
				];

				if ( $args['blockSupports'] && ( $args['blockSupports']['background'] ?? false ) ) {
					$block_atts['supports']['color']                          = [];
					$block_atts['supports']['color']['__experimentalDuotone'] = '> .bb\:block-background';
				}

				global $wp_version;
				if ( version_compare( $wp_version, '5.9', '>=' ) ) {
					$block_atts['view_script'] = 'boldblocks-custom-blocks-frontend';
				} else {
					$block_atts['script'] = 'boldblocks-custom-blocks-frontend';
				}

				// Enqueue scripts for specific block types.
				if ( isset( $args['parent'] ) && isset( $args['parent']['layoutType'] ) && 'carousel' === $args['parent']['layoutType'] ) {
					// Mark it as carousel item block.
					$block_atts['supports']['parentLayoutType'] = 'carousel';

					wp_enqueue_block_style(
						$args['parent']['name'],
						[ 'handle' => 'boldblocks-carousel-frontend' ]
					);
				}

				register_block_type( $name, $block_atts );
			}
		}

		/**
		 * Register code editor scripts
		 *
		 * @return void
		 */
		public function register_code_editor_scripts() {
			$screen = get_current_screen();
			if ( $screen && $screen->is_block_editor ) {
				wp_enqueue_code_editor( array( 'type' => 'text/css' ) );
			}
		}

		/**
		 * Register scripts
		 *
		 * @return void
		 */
		public function register_scripts() {
			// Custom blocks asset file.
			$custom_blocks_asset = $this->the_plugin_instance->include_file( "build/custom-blocks{$this->the_plugin_instance->get_script_suffix()}.asset.php" );

			// Scripts.
			wp_register_script(
				'boldblocks-custom-blocks',
				$this->the_plugin_instance->get_file_uri( "build/custom-blocks{$this->the_plugin_instance->get_script_suffix()}.js" ),
				array_merge( $custom_blocks_asset['dependencies'] ?? [], array( 'code-editor', 'csslint' ) ),
				$this->the_plugin_instance->get_script_version( $custom_blocks_asset ),
				true
			);

			// Add translation.
			wp_set_script_translations( 'boldblocks-custom-blocks', 'content-blocks-builder' );

			// Define localize sripts.
			$blocks_scripts = [
				'blocks'        => $this->get_all_custom_blocks(),
				'allowedBlocks' => $this->get_custom_blocks_allowed_blocks(),
			];

			// Load all custom blocks for registration.
			wp_localize_script(
				'boldblocks-custom-blocks',
				'BoldblocksBlocks',
				$blocks_scripts
			);

			// Styles.
			wp_register_style(
				'boldblocks-custom-blocks',
				$this->the_plugin_instance->get_file_uri( "build/custom-blocks{$this->the_plugin_instance->get_script_suffix()}.css" ),
				[],
				$this->the_plugin_instance->get_script_version( $custom_blocks_asset )
			);

			// Custom blocks asset file.
			$custom_blocks_frontend_asset = $this->the_plugin_instance->include_file( "build/custom-blocks-frontend{$this->the_plugin_instance->get_script_suffix()}.asset.php" );

			// Frontend script.
			wp_register_script(
				'boldblocks-custom-blocks-frontend',
				$this->the_plugin_instance->get_file_uri( "build/custom-blocks-frontend{$this->the_plugin_instance->get_script_suffix()}.js" ),
				$custom_blocks_frontend_asset['dependencies'] ?? [],
				$this->the_plugin_instance->get_script_version( $custom_blocks_frontend_asset ),
				true
			);

			// Add translation.
			wp_set_script_translations( 'boldblocks-custom-blocks-frontend', 'content-blocks-builder' );

			// Carousel asset file.
			$caousel_frontend_asset = $this->the_plugin_instance->include_file( 'build/carousel-frontend.asset.php' );

			// Carousel styles.
			wp_register_style(
				'boldblocks-carousel-frontend',
				$this->the_plugin_instance->get_file_uri( 'build/carousel-frontend.css' ),
				[],
				$this->the_plugin_instance->get_script_version( $caousel_frontend_asset )
			);

			// Carousel frontend script.
			wp_register_script(
				'boldblocks-carousel-frontend',
				$this->the_plugin_instance->get_file_uri( 'build/carousel-frontend.js' ),
				$caousel_frontend_asset['dependencies'] ?? [],
				$this->the_plugin_instance->get_script_version( $caousel_frontend_asset ),
				true
			);

			// Add translation.
			wp_set_script_translations( 'boldblocks-carousel-frontend', 'content-blocks-builder' );
		}

		/**
		 * Get all custom blocks.
		 *
		 * @return array
		 */
		public function get_all_custom_blocks() {
			if ( ! $this->all_custom_blocks ) {
				$this->all_custom_blocks = $this->get_posts();
			}

			return $this->all_custom_blocks;
		}

		/**
		 * Query posts and cache the results.
		 *
		 * @param bool $force_refresh Optional. Whether to force the cache to be refreshed. Default false.
		 * @return array Array of WP_Post objects.
		 */
		public function get_posts( $force_refresh = false ) {
			$cache_key = 'bb_block_posts';

			$posts = get_transient( $cache_key );

			// If nothing is found, build the object.
			if ( true === $force_refresh || false === $posts ) {
				// Query posts.
				$posts = $this->query_posts();

				if ( ! is_wp_error( $posts ) && count( $posts ) > 0 ) {
					set_transient( $cache_key, $posts, HOUR_IN_SECONDS );
				}
			}

			return $posts;
		}

		/**
		 * Query and parse blocks
		 *
		 * @return array
		 */
		public function query_posts() {
			$post_args = [
				'post_type'      => 'boldblocks_block',
				'posts_per_page' => $this->max_items,
				'orderby'        => [
					'menu_order' => 'DESC',
					'date'       => 'DESC',
				],
			];

			$raw_posts = get_posts( $post_args );

			$block_posts = array_map(
				function( $item ) {
					// Template lock.
					$template_lock = get_post_meta( $item->ID, 'boldblocks_template_lock', true );
					// Convert boolean string to boolean.
					$template_lock = 'false' === $template_lock ? false : $template_lock;

					$block_args = [
						'id'                    => $item->ID,
						'name'                  => sprintf( 'boldblocks/%s', sanitize_key( $item->post_name ) ),
						'title'                 => wp_strip_all_tags( $item->post_title ),
						'postContent'           => $item->post_content,
						'blocks'                => get_post_meta( $item->ID, 'boldblocks_block_blocks', true ),
						'description'           => get_post_meta( $item->ID, 'boldblocks_block_description', true ),
						'templateLock'          => $template_lock,
						'className'             => get_post_meta( $item->ID, 'boldblocks_block_class', true ),
						'blockIcon'             => get_post_meta( $item->ID, 'boldblocks_block_icon', true ),
						'blockSupports'         => get_post_meta( $item->ID, 'boldblocks_block_supports', true ),
						'enableVariationPicker' => ! ! get_post_meta( $item->ID, 'boldblocks_enable_variation_picker', true ),
						'blockVersion'          => get_post_meta( $item->ID, 'boldblocks_block_version', true ),
						'dependentBlocks'       => get_post_meta( $item->ID, 'boldblocks_block_dependent_blocks', true ),
					];

					// Get parent block parameters.
					$enable_repeater = get_post_meta( $item->ID, 'boldblocks_enable_repeater', true );
					if ( $enable_repeater ) {
						$parent_name = get_post_meta( $item->ID, 'boldblocks_parent_block_name', true );
						if ( ! $parent_name ) {
							$parent_name = $item->post_name;
						}

						$parent_layout_type = get_post_meta( $item->ID, 'boldblocks_parent_layout_type', true );

						$parent_title = get_post_meta( $item->ID, 'boldblocks_parent_block_title', true );
						if ( ! $parent_title ) {
							// translators: Repeater block title.
							$parent_title = sprintf( __( 'Repeater: %s', 'content-blocks-builder' ), $item->post_title );
						}

						$parent_args = [
							'name'                  => sprintf( 'boldblocks/%s-repeater', sanitize_key( $parent_name ) ),
							'title'                 => wp_strip_all_tags( $parent_title ),
							'description'           => get_post_meta( $item->ID, 'boldblocks_parent_block_description', true ),
							'className'             => get_post_meta( $item->ID, 'boldblocks_parent_block_class', true ) ?? '',
							'blockIcon'             => get_post_meta( $item->ID, 'boldblocks_parent_block_icon', true ),
							'blockSupports'         => get_post_meta( $item->ID, 'boldblocks_parent_block_supports', true ),
							'enableVariationPicker' => ! ! get_post_meta( $item->ID, 'boldblocks_parent_enable_variation_picker', true ),
						];

						$is_fixed_nested_item_count = get_post_meta( $item->ID, 'boldblocks_is_fixed_nested_item_count', true );
						if ( $is_fixed_nested_item_count ) {
							$nested_item_count              = get_post_meta( $item->ID, 'boldblocks_nested_item_count', true );
							$nested_item_count              = $nested_item_count ? absint( $nested_item_count ) : 1;
							$parent_args['nestedItemCount'] = $nested_item_count ? $nested_item_count : 1;
						}

						$parent_args['layoutType'] = $parent_layout_type;
						$block_args['parent']      = $parent_args;

						$block_args['standalone'] = ! get_post_meta( $item->ID, 'boldblocks_disable_standalone', true );
					}

					return $block_args;
				},
				$raw_posts
			);

			return $block_posts;
		}

		/**
		 * Clear transient cache
		 *
		 * @return void
		 */
		public function clear_transient_cache() {
			delete_transient( 'bb_block_posts' );
		}

		/**
		 * Add custom blocks to the list of blocks that can create patterns from.
		 *
		 * @param array $allowed_blocks
		 * @return array
		 */
		public function allow_creating_patterns( $allowed_blocks ) {
			return array_merge( $allowed_blocks, array_keys( $this->custom_blocks ), array_keys( $this->repeater_blocks ) );
		}

		/**
		 * Get list of blocks that allow creating custom blocks from toolbar menu.
		 */
		public function get_custom_blocks_allowed_blocks() {
			return apply_filters(
				'boldblocks/get_custom_block_allowed_blocks',
				[
					'core/group',
					'core/cover',
					'core/columns',
					'core/media-text',
				]
			);
		}

		/**
		 * Add custom columns to custom post type
		 *
		 * @param  array $columns The array of columns.
		 * @return array
		 */
		public function add_boldblocks_block_custom_columns( $columns ) {
			$custom_columns = array(
				'block_icon'        => esc_html__( 'Block icon', 'content-blocks-builder' ),
				'block_name'        => esc_html__( 'Block name', 'content-blocks-builder' ),
				'parent_block_icon' => esc_html__( 'Parent icon', 'content-blocks-builder' ),
				'parent_block_name' => esc_html__( 'Parent block name', 'content-blocks-builder' ),
			);

			if ( defined( 'CBB_DEBUG' ) && CBB_DEBUG ) {
				$custom_columns['block_version'] = esc_html__( 'Block version', 'content-blocks-builder' );
			}

			// Add after title column.
			$return = array_slice( $columns, 0, 2, true ) + $custom_columns + array_slice( $columns, 2, count( $columns ) - 2, true );

			return $return;
		}

		/**
		 * Manage custom columns.
		 *
		 * @param string $column  the column name.
		 * @param int    $post_id the post id.
		 */
		public function manage_boldblocks_block_columns( $column, $post_id ) {
			switch ( $column ) {
				case 'block_icon':
					$block_icon = get_post_meta( $post_id, 'boldblocks_block_icon', true );
					if ( empty( $block_icon ) ) {
						$block_icon = '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" class="bb-icon bb-icon--block-default"><path d="M19 8h-1V6h-5v2h-2V6H6v2H5c-1.1 0-2 .9-2 2v8c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2v-8c0-1.1-.9-2-2-2zm.5 10c0 .3-.2.5-.5.5H5c-.3 0-.5-.2-.5-.5v-8c0-.3.2-.5.5-.5h14c.3 0 .5.2.5.5v8z"/></svg>';
					}
					echo $block_icon; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					break;

				case 'block_name':
					echo esc_html( 'boldblocks/' . sanitize_key( get_post_field( 'post_name', $post_id ) ) );
					break;

				case 'parent_block_icon':
					if ( get_post_meta( $post_id, 'boldblocks_enable_repeater', true ) ) {
						$block_icon = get_post_meta( $post_id, 'boldblocks_parent_block_icon', true );
						if ( empty( $block_icon ) ) {
							$block_icon = '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" class="bb-icon bb-icon--block-default"><path d="M19 8h-1V6h-5v2h-2V6H6v2H5c-1.1 0-2 .9-2 2v8c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2v-8c0-1.1-.9-2-2-2zm.5 10c0 .3-.2.5-.5.5H5c-.3 0-.5-.2-.5-.5v-8c0-.3.2-.5.5-.5h14c.3 0 .5.2.5.5v8z"/></svg>';
						}
						echo $block_icon; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					} else {
						esc_html_e( 'NA', 'content-blocks-builder' );
					}
					break;

				case 'parent_block_name':
					$enable_repeater = get_post_meta( $post_id, 'boldblocks_enable_repeater', true );
					if ( $enable_repeater ) {
						echo esc_html( 'boldblocks/' . sanitize_key( get_post_field( 'post_name', $post_id ) ) . '-repeater' );
					} else {
						esc_html_e( 'NA', 'content-blocks-builder' );
					}
					break;

				case 'block_version':
					$block_version = get_post_meta( $post_id, 'boldblocks_block_version', true );
					if ( $block_version ) {
						echo esc_html( $block_version );
					} else {
						esc_html_e( '1.x', 'content-blocks-builder' );
					}
					break;

				// Just break out of the switch statement for everything else.
				default:
					break;
			}
		}

		/**
		 * Add style to custom columns
		 *
		 * @return void
		 */
		public function add_custom_columns_style() {
			echo '<style>.column-block_icon, .column-parent_block_icon {width: 70px;}.column-block_icon svg,.column-parent_block_icon svg {max-width: 32px;max-height: 32px;}</style>';
		}

		/**
		 * Add custom block's meta fields to meta revisioning.
		 *
		 * @param array $fields
		 * @return array
		 */
		public function add_fields_to_revision_meta_keys( $fields ) {
			return array_merge(
				$fields,
				[
					'boldblocks_block_blocks',
					'boldblocks_block_class',
					'boldblocks_block_description',
					'boldblocks_block_icon',
					'boldblocks_block_supports',
					'boldblocks_template_lock',
					'boldblocks_enable_variation_picker',
					'boldblocks_enable_repeater',
					'boldblocks_parent_layout_type',
					'boldblocks_parent_block_icon',
					'boldblocks_parent_block_supports',
					'boldblocks_parent_enable_variation_picker',
					'boldblocks_disable_standalone',
					'boldblocks_is_fixed_nested_item_count',
					'boldblocks_nested_item_count',
					'boldblocks_parent_block_name',
					'boldblocks_parent_block_title',
					'boldblocks_parent_block_description',
					'boldblocks_parent_block_class',
				]
			);
		}

		/**
		 * Add theme name to body class
		 *
		 * @param array $classes
		 * @return array
		 */
		public function add_body_class( $classes ) {
			// Add theme slug.
			$classes[] = get_stylesheet();

			return $classes;
		}

		/**
		 * Save the plugin version to custom blocks
		 *
		 * @param int     $post_id
		 * @param WP_Post $post
		 * @param boolean $update
		 * @return void
		 */
		public function save_version_to_block( $post_id, $post, $update ) {
			// Bail if it is a update action.
			if ( $update ) {
				return;
			}

			// Save the plugin version to blocks.
			update_post_meta( $post_id, 'boldblocks_block_version', $this->the_plugin_instance->get_plugin_version() );
		}

		/**
		 * Prevent users from deleting core blocks.
		 *
		 * @param array $allcaps
		 * @param array $caps
		 * @param array $args
		 * @return array
		 */
		public function prevent_core_blocks_deletion( $allcaps, $caps, $args ) {
			// Bail out if we're not asking about deleting a post.
			if ( 'delete_post' !== $args[0] ) {
				return $allcaps;
			}

			// Bail out for users who can't delete posts.
			if ( empty( $allcaps['delete_posts'] ) ) {
				return $allcaps;
			}

			// Load the post data.
			$post = get_post( $args[2] );

			// Bail out if the post type is not boldblocks_block.
			if ( 'boldblocks_block' !== $post->post_type ) {
				return $allcaps;
			}

			// Don't allow deleting core blocks.
			if ( \in_array( $post->post_name, [ 'group', 'carousel-item', 'grid-item', 'stack-item', 'accordion-item' ], true ) ) {
				$allcaps[ $caps[0] ] = false;
			}

			return $allcaps;
		}

		/**
		 * Placeholder text. Default 'Add title'.
		 *
		 * @param string $text
		 * @return string
		 */
		public function enter_title_here( $text ) {
			$post_type = get_post_type();
			if ( 'boldblocks_block' === $post_type ) {
				$text = __( 'Add block title', 'content-blocks-builder' );
			}

			return $text;
		}

		/**
		 * Enqueue frontend scripts for carousel layout in the query loop block
		 *
		 * @param string   $block_content
		 * @param array    $block
		 * @param WP_Block $block_instance
		 * @return string
		 */
		public function enqueue_frontend_carousel_scripts( $block_content, $block, $block_instance ) {
			// Ignore admin side.
			if ( is_admin() ) {
				return $block_content;
			}

			if ( isset( $GLOBALS['boldblocks_carousel_loaded'] ) ) {
				return $block_content;
			}

			// Make sure we have the blockName.
			if ( empty( $block['blockName'] ) ) {
				return $block_content;
			}

			if ( 'core/query' === $block['blockName'] ) {
				// If this is a core/query block, enqueue the carousel script.
				if ( ! empty( $block['attrs'] ) && ! empty( $block['attrs']['displayLayout'] ) && 'carousel' === $block['attrs']['displayLayout']['type'] ?? '' ) {
					wp_enqueue_script( 'boldblocks-carousel-frontend' );
					wp_enqueue_style( 'boldblocks-carousel-frontend' );

					$GLOBALS['boldblocks_carousel_loaded'] = true;
				}
			} elseif ( strpos( $block['blockName'], 'boldblocks/' ) === 0 ) {
				if ( isset( $block_instance->block_type->supports['parentLayoutType'] ) && 'carousel' === $block_instance->block_type->supports['parentLayoutType'] ) {
					wp_enqueue_script( 'boldblocks-carousel-frontend' );
					wp_enqueue_style( 'boldblocks-carousel-frontend' );

					$GLOBALS['boldblocks_carousel_loaded'] = true;
				}
			}

			return $block_content;
		}
	}
endif;
