<?php
/**
 * The custom style component
 *
 * @package   BoldBlocks
 * @author    Phi Phan <mrphipv@gmail.com>
 * @copyright Copyright (c) 2022, Phi Phan
 */

namespace BoldBlocks;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'CustomStyle' ) ) :
	/**
	 * The controller class for style.
	 */
	class CustomStyle extends CoreComponent {
		/**
		 * Run main hooks
		 *
		 * @return void
		 */
		public function run() {
			// Render block style.
			add_filter( 'render_block', [ $this, 'render_block_style' ], 10, 2 );

			// Build dynamic background.
			add_filter( 'render_block', [ $this, 'build_dynamic_background' ], 10, 3 );
		}

		/**
		 * Render style for all supported blocks
		 *
		 * @param string $block_content
		 * @param array  $block
		 * @return string
		 */
		public function render_block_style( $block_content, $block ) {
			$custom_style = $this->get_custom_css_style( $block );

			// Bail if the block has no style.
			if ( ! $custom_style ) {
				return $block_content;
			}

			// Buil selector.
			$selector = "b-{$custom_style['id']}";

			// Get custom style.
			$style       = $custom_style['value'];
			$style       = preg_replace( '/(<([^>]+)>)/mi', '', $style );
			$block_style = str_replace( 'selector', ".{$selector}", $style );

			if ( empty( $block_style ) ) {
				return $block_content;
			}

			// Add selector to block wrapper element.
			$content = $this->add_class_to_block( $block_content, $selector );

			return $content . '<style>' . $block_style . '</style>';
		}

		/**
		 * Check whether current block has custom style or not.
		 *
		 * @param [type] $block
		 * @return boolean
		 */
		private function get_custom_css_style( $block ) {
			// There is no boldblocks value.
			if ( ! isset( $block['attrs']['boldblocks'] ) || ! isset( $block['attrs']['boldblocks']['customCSS'] ) ) {
				return false;
			}

			$custom_css = $block['attrs']['boldblocks']['customCSS'];
			if ( ! is_array( $custom_css ) || empty( $custom_css['id'] ) || empty( $custom_css['value'] ) ) {
				return false;
			}

			return $custom_css;
		}

		/**
		 * Render dynamic background image
		 *
		 * @param string   $block_content
		 * @param array    $block
		 * @param WP_Block $block_instance
		 * @return string
		 */
		public function build_dynamic_background( $block_content, $block, $block_instance ) {
			// There is no background value.
			if ( ! ( $block['attrs']['boldblocks']['background'] ?? false ) ) {
				return $block_content;
			}

			$background = $block['attrs']['boldblocks']['background'];
			$media_type = $background['mediaType'] ?? false;
			if ( $media_type === 'video' || ! ( $background['image']['useFeaturedImage'] ?? false ) ) {
				return $block_content;
			}

			// Has custom background image.
			if ( $background['image']['url'] ?? false ) {
				return $block_content;
			}

			$post_id = $block_instance->context['postId'] ?? false;
			if ( ! $post_id || ! has_post_thumbnail( $post_id ) ) {
				return $block_content;
			}

			$settings            = $background['settings'] ?? [];
			$image_element       = '';
			$dataset             = [];
			$background_position = round( ( $settings['focalPoint']['x'] ?? .5 ) * 100 ) . '% ' . round( ( $settings['focalPoint']['y'] ?? .5 ) * 100 ) . '%';

			$thumnail_url      = get_the_post_thumbnail_url( $post_id, 'full' );
			$alt_text          = $background['image']['customAlt'] ?? get_the_post_thumbnail_caption( $post_id );
			$img_style         = 'width:100%;height:100%;object-fit: cover;object-position:' . $background_position . ';';
			$background_styles = [];
			$animation_type    = false;
			if ( ! empty( $settings['isImgElement'] ) ) {
				$image_element = sprintf( '<img src="%1$s" alt="%2$s" style="%3$s" class="%4$s"/>', esc_url( $thumnail_url ), esc_attr( $alt_text ), esc_attr( $img_style ), 'bb:block-background--img' );
			} else {
				$background_styles = [
					'background-image'    => 'url(' . $thumnail_url . ')',
					'background-position' => $background_position,
				];
				if ( ! empty( $settings['isFixed'] ) ) {
					$background_styles['background-attachment'] = 'fixed';
				}

				$background_styles['background-repeat'] = $settings['repeat'] ?? 'no-repeat';
				$background_styles['background-size']   = $settings['size'] ?? 'cover';

				$animation_type = $settings['animation']['type'] ?? false;
				if ( $animation_type && 'none' !== $animation_type ) {
					if ( in_array( $animation_type, [ 'bg-scroll-horizontal', 'bg-scroll-vertical' ], true ) ) {
						if ( $settings['animation']['settings']['duration'] ?? false ) {
							$background_styles['animation-duration'] = $settings['animation']['settings']['duration'] . 's';
						}

						if ( $background_styles['background-repeat'] !== 'repeat' ) {
							$background_styles['background-repeat'] = 'bg-scroll-horizontal' === $animation_type ? 'repeat-x' : 'repeat-y';
						}
					} elseif ( 'bg-parallax' === $animation_type ) {
						unset( $background_styles['background-attachment'] );

						$dataset['data-speed'] = $settings['animation']['settings']['speed'] ?? 0.5;

						if ( $settings['animation']['settings']['opacity'] ?? false ) {
							$dataset['data-opacity'] = 'true';
						}
					}
				}
			}

			$classes = [ 'bb:block-background bb:block-background--image' ];
			if ( $animation_type && 'none' !== $animation_type ) {
				$classes[] = $animation_type;
			}

			if ( in_array( $animation_type, [ 'bg-scroll-horizontal', 'bg-scroll-vertical' ], true ) && ( $settings['animation']['settings']['reverseDirection'] ?? false ) ) {
				$classes[] = 'is-reverse';
			}

			$background_style = $background_styles ? array_reduce(
				array_keys( $background_styles ),
				function ( $carry, $key ) use ( $background_styles ) {
					return $carry . $key . ':' . $background_styles[ $key ] . ';';
				},
				''
			) : '';

			$data_attribute = $dataset ? array_reduce(
				array_keys( $dataset ),
				function ( $carry, $key ) use ( $dataset ) {
					return $carry . ' ' . $key . '="' . $dataset[ $key ] . '"';
				},
				' '
			) : '';

			$background_image = sprintf( '<div class="%1$s" style="%2$s"%3$s>%4$s</div>', \implode( ' ', $classes ), $background_style, $data_attribute, $image_element );

			$block_class = 'bb:has-background bb:has-background--image';
			if ( $animation_type && 'none' !== $animation_type ) {
				$block_class .= ' js-animation-' . $animation_type;
			}

			$content = $this->add_class_to_block( $block_content, $block_class );

			$content = \preg_replace(
				'/' . \preg_quote( '>', '/' ) . '/',
				'>' . $background_image,
				$content,
				1
			);

			return $content;
		}

		/**
		 * Check wether the block markup has a attribute or not.
		 *
		 * @param string $attribute
		 * @param string $block_content
		 * @return boolean
		 */
		private function block_has_attribute( $attribute, $block_content ) {
			$greater_position = \strpos( $block_content, '>' );
			if ( $greater_position !== false ) {
				$block_tag = \substr( $block_content, 0, $greater_position + 1 );

				return \strpos( $block_tag, $attribute . '=' );
			}

			return false;
		}

		/**
		 * Add CSS class to block wrapper
		 *
		 * @param string $block_content
		 * @param string $classes
		 * @return string
		 */
		private function add_class_to_block( $block_content, $classes ) {
			if ( ! $this->block_has_attribute( 'class', $block_content ) ) {
				$content = \preg_replace(
					'/' . \preg_quote( '>', '/' ) . '/',
					'class="' . $classes . '">',
					$block_content,
					1
				);
			} else {
				$content = \preg_replace(
					'/' . \preg_quote( 'class="', '/' ) . '/',
					'class = "' . $classes . ' ',
					$block_content,
					1
				);
			}

			return $content;
		}
	}
endif;
