<?php
/**
 * The icon library
 *
 * @package   BoldBlocks
 * @author    Phi Phan <mrphipv@gmail.com>
 * @copyright Copyright (c) 2022, Phi Phan
 */

namespace BoldBlocks;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'IconLibrary' ) ) :
	/**
	 * The controller class for icon library.
	 */
	class IconLibrary extends CoreComponent {
		/**
		 * Run main hooks
		 *
		 * @return void
		 */
		public function run() {
			// Add rest api endpoint to query icon library.
			add_action( 'rest_api_init', [ $this, 'register_icon_library_endpoint' ] );

			// Load data for js.
			// add_action( 'enqueue_block_editor_assets', [ $this, 'enqueue_block_editor_assets' ] );

			if ( apply_filters( 'boldblocks_support_svg_image', true ) ) {
				// Allow SVG upload.
				add_filter( 'upload_mimes', [ $this, 'mime_types_support_svg' ] );

				// Display svg images.
				add_action( 'admin_head', [ $this, 'display_svg_thumb' ] );

				// Add metadata to svg images.
				add_filter( 'wp_update_attachment_metadata', [ $this, 'update_svg_metadata' ], 10, 2 );
			}
		}

		/**
		 * Enqueue data for js
		 *
		 * @return void
		 */
		public function enqueue_block_editor_assets() {
			$this->enqueue_localize_scripts( [ 'boldblocks-custom-blocks', 'boldblocks-variations' ] );
		}

		/**
		 * Enqueue localize scripts
		 *
		 * @return void
		 */
		public function enqueue_localize_scripts( $handles ) {
			// icons_version file path.
			$icons_version_file = $this->the_plugin_instance->get_file_path( 'data/icon-library/icons-version.json' );

			// Bail if ther is no icons-version.json.
			if ( \file_exists( $icons_version_file ) ) {
				$icons_version = \file_get_contents( $icons_version_file );
				$icons_version = \json_decode( $icons_version, true );

				// Define localize sripts.
				$localization_scripts = [
					'iconsVersion' => $icons_version['version'] ?? '1.0.0',
				];

				// Register localize scripts.
				foreach ( $handles as $handle ) {
					wp_localize_script(
						$handle,
						'IconLibrary',
						$localization_scripts
					);
				}
			}
		}

		/**
		 * Build a custom endpoint to query icon library.
		 *
		 * @return void
		 */
		public function register_icon_library_endpoint() {
			register_rest_route(
				'boldblocks/v1',
				'/getIconLibrary/',
				array(
					'methods'             => 'GET',
					'callback'            => [ $this, 'get_icon_library' ],
					'permission_callback' => function () {
						return current_user_can( 'publish_posts' );
					},
				)
			);
		}

			/**
			 * Get icon library.
			 *
			 * @param WP_REST_Request $request The request object.
			 * @return void
			 */
		public function get_icon_library( $request ) {
			// icons file path.
			$icons_file = $this->the_plugin_instance->get_file_path( 'data/icon-library/icons.json' );

			// Send the error if the icons file is not exists.
			if ( ! \file_exists( $icons_file ) ) {
				wp_send_json_error( __( 'The icons.json file is not exists.', 'content-blocks-builder' ), 500 );
			}

			// Parse json.
			$icons = wp_json_file_decode( $icons_file, [ 'associative' => true ] );

			wp_send_json(
				[
					'data'    => $icons,
					'success' => true,
				]
			);
		}

		/**
		 * Add SVG mine types
		 *
		 * @param array $mimes
		 * @return array
		 */
		public function mime_types_support_svg( $mimes ) {
			$mimes['svg'] = 'image/svg+xml';

			return $mimes;
		}

		/**
		 * Display SVG images
		 *
		 * @return void
		 */
		public function display_svg_thumb() {
			echo '<style>
				.wp-list-table .media-icon img[src$=".svg"], .editor-post-featured-image img[src$=".svg"] {
					width: 100% !important;
					height: auto !important;
				}
			</style>';
		}

		/**
		 * Generate width/height for uploaded SVG images
		 * https://css-tricks.com/snippets/wordpress/allow-svg-through-wordpress-media-uploader/#comment-1606112
		 *
		 * @param array $data metadata for uploaded image
		 * @param int   $id the attachment id
		 * @return void
		 */
		public function update_svg_metadata( $data, $id ) {
			// Filter makes sure that the post is an attachment.
			$attachment = get_post( $id );

			// The attachment mime_type.
			$mime_type = $attachment->post_mime_type;

			// If the attachment is an svg.
			if ( 'image/svg+xml' === $mime_type ) {
				// If the svg metadata are empty or the width is empty or the height is empty, then get the attributes from xml.
				if ( empty( $data ) || empty( $data['width'] ) || empty( $data['height'] ) ) {
					$xml = simplexml_load_file( get_attached_file( $id ) );
					if ( $xml ) {
						$attr          = $xml->attributes();
						$viewbox       = explode( ' ', $attr->viewBox );
						$data['width'] = isset( $attr->width ) && preg_match( '/\d+/', $attr->width, $value ) ? (int) $value[0] : ( count( $viewbox ) == 4 ? (int) $viewbox[2] : null );
						if ( intval( $data['width'] ) === 0 ) {
							$data['width'] = 1;
						}
						$data['height'] = isset( $attr->height ) && preg_match( '/\d+/', $attr->height, $value ) ? (int) $value[0] : ( count( $viewbox ) == 4 ? (int) $viewbox[3] : null );
						if ( intval( $data['height'] ) === 0 ) {
							$data['height'] = 1;
						}
					}
				}
			}

			return $data;
		}
	}
endif;
