<?php
/**
 * Register custom patterns
 *
 * @package    BoldBlocks
 * @author     Phi Phan <mrphipv@gmail.com>
 * @copyright  Copyright (c) 2022, Phi Phan
 */

namespace BoldBlocks;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'Patterns' ) ) :
	/**
	 * Manage custom patterns.
	 */
	class Patterns extends CoreComponent {
		/**
		 * The pattern library url
		 *
		 * @var string
		 */
		private $pattern_library_url = 'https://boldpatterns.net';

		/**
		 * Store all custom patterns
		 *
		 * @var array
		 */
		protected $all_custom_patterns = [];

		/**
		 * Max items that can query
		 *
		 * @var integer
		 */
		protected $max_items = 200;

		/**
		 * Run main hooks
		 *
		 * @return void
		 */
		public function run() {
			// Register custom post type.
			add_action( 'init', [ $this, 'register_custom_post_type' ], 5 );

			// Load data for js.
			add_action( 'enqueue_block_editor_assets', [ $this, 'enqueue_block_editor_assets' ] );

			// Register patterns.
			add_action( 'init', [ $this, 'register_patterns' ] );

			// Custom pattern columns.
			add_filter( 'manage_edit-boldblocks_pattern_columns', [ $this, 'add_pattern_custom_columns' ] );
			add_action( 'manage_boldblocks_pattern_posts_custom_column', [ $this, 'manage_pattern_columns' ], 10, 2 );

			// Add custom admin menus.
			add_action( 'admin_menu', [ $this, 'add_admin_menu' ] );

			// Add rest api endpoint to query all patterns.
			add_action( 'rest_api_init', [ $this, 'register_get_patterns_endpoint' ] );

			// Add rest api endpoint to query all pattern keywords.
			add_action( 'rest_api_init', [ $this, 'register_get_pattern_keywords_endpoint' ] );

			// Add rest api endpoint to query all pattern categories.
			add_action( 'rest_api_init', [ $this, 'register_get_pattern_categories_endpoint' ] );

			// Register rest fields.
			add_action( 'rest_api_init', [ $this, 'register_rest_fields' ] );

			// Add meta fields to meta revisioning.
			add_filter( 'boldblocks_post_revision_meta_keys', [ $this, 'add_fields_to_revision_meta_keys' ] );

			// Change the placeholder text.
			add_filter( 'enter_title_here', [ $this, 'enter_title_here' ] );

			// Clear transient cache.
			add_action( 'save_post_boldblocks_pattern', [ $this, 'clear_transient_cache' ] );
		}

		/**
		 * Register custom post type
		 *
		 * @return void
		 */
		public function register_custom_post_type() {
			// Block pattern args.
			$pattern_labels = array(
				'name'               => _x( 'Block Patterns', 'post type general name', 'content-blocks-builder' ),
				'singular_name'      => _x( 'Block Pattern', 'post type singular name', 'content-blocks-builder' ),
				'menu_name'          => _x( 'Block Patterns', 'admin menu', 'content-blocks-builder' ),
				'name_admin_bar'     => _x( 'Block Patterns', 'add new on admin bar', 'content-blocks-builder' ),
				'add_new'            => _x( 'Add New', 'item', 'content-blocks-builder' ),
				'add_new_item'       => __( 'Add New Block Pattern', 'content-blocks-builder' ),
				'new_item'           => __( 'New Block Pattern', 'content-blocks-builder' ),
				'edit_item'          => __( 'Edit Block Pattern', 'content-blocks-builder' ),
				'view_item'          => __( 'View Block Pattern', 'content-blocks-builder' ),
				'all_items'          => __( 'All Patterns', 'content-blocks-builder' ),
				'search_items'       => __( 'Search Block Patterns', 'content-blocks-builder' ),
				'parent_item_colon'  => __( 'Parent Block Pattern:', 'content-blocks-builder' ),
				'not_found'          => __( 'No items found.', 'content-blocks-builder' ),
				'not_found_in_trash' => __( 'No items found in trash.', 'content-blocks-builder' ),
			);

			$pattern_args = array(
				'labels'              => $pattern_labels,
				'description'         => __( 'All custom block patterns', 'content-blocks-builder' ),
				'public'              => true,
				'publicly_queryable'  => false,
				'exclude_from_search' => true,
				'show_ui'             => true,
				'show_in_menu'        => 'edit.php?post_type=boldblocks_block',
				'show_in_nav_menus'   => false,
				'show_in_admin_bar'   => true,
				'show_in_rest'        => true,
				'rest_base'           => 'boldblocks-patterns',
				'query_var'           => false,
				'rewrite'             => false,
				'capability_type'     => 'post',
				'has_archive'         => false,
				'hierarchical'        => false,
				'menu_position'       => 21,
				'menu_icon'           => 'dashicons-editor-kitchensink',
				'supports'            => array( 'title', 'editor', 'author', 'custom-fields', 'revisions', 'page-attributes' ),
				'can_export'          => true,
			);

			// Register post type.
			register_post_type( 'boldblocks_pattern', $pattern_args );

			// Pattern keywords.
			$pattern_keywords_labels = array(
				'name'                       => _x( 'Keywords', 'Taxonomy General Name', 'content-blocks-builder' ),
				'singular_name'              => _x( 'Keyword', 'Taxonomy Singular Name', 'content-blocks-builder' ),
				'menu_name'                  => __( 'Keywords', 'content-blocks-builder' ),
				'all_items'                  => __( 'All Items', 'content-blocks-builder' ),
				'new_item_name'              => __( 'New Item Name', 'content-blocks-builder' ),
				'add_new_item'               => __( 'Add New Keyword', 'content-blocks-builder' ),
				'edit_item'                  => __( 'Edit Item', 'content-blocks-builder' ),
				'update_item'                => __( 'Update Item', 'content-blocks-builder' ),
				'view_item'                  => __( 'View Item', 'content-blocks-builder' ),
				'separate_items_with_commas' => __( 'Separate items with commas', 'content-blocks-builder' ),
				'add_or_remove_items'        => __( 'Add or remove items', 'content-blocks-builder' ),
				'choose_from_most_used'      => __( 'Choose from the most used', 'content-blocks-builder' ),
				'popular_items'              => __( 'Popular Items', 'content-blocks-builder' ),
				'search_items'               => __( 'Search Items', 'content-blocks-builder' ),
				'not_found'                  => __( 'Not Found', 'content-blocks-builder' ),
			);

			$pattern_keywords_args = array(
				'labels'            => $pattern_keywords_labels,
				'hierarchical'      => false,
				'public'            => false,
				'show_ui'           => true,
				'show_admin_column' => true,
				'show_in_nav_menus' => false,
				'show_tagcloud'     => false,
				'query_var'         => false,
				'show_in_rest'      => true,
			);

			register_taxonomy( 'boldblocks_pattern_keywords', array( 'boldblocks_pattern' ), $pattern_keywords_args );

			// Meta fields.
			register_meta(
				'post',
				'boldblocks_pattern_categories',
				array(
					'object_subtype' => 'boldblocks_pattern',
					'type'           => 'array',
					'show_in_rest'   => array(
						'schema' => array(
							'items' => array(
								'type'       => 'object',
								'properties' => array(
									'name'  => array(
										'type' => 'string',
									),
									'label' => array(
										'type' => 'string',
									),
								),
							),
						),
					),
					'default'        => [],
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_pattern_description',
				array(
					'object_subtype' => 'boldblocks_pattern',
					'type'           => 'string',
					'show_in_rest'   => true,
					'default'        => '',
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_pattern_dependencies',
				array(
					'object_subtype' => 'boldblocks_pattern',
					'type'           => 'array',
					'show_in_rest'   => array(
						'schema' => array(
							'items' => array(
								'type'       => 'object',
								'properties' => array(
									'slug' => array(
										'type' => 'string',
									),
									'name' => array(
										'type' => 'string',
									),
								),
							),
						),
					),
					'default'        => [],
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_pattern_dependent_blocks',
				array(
					'object_subtype' => 'boldblocks_pattern',
					'show_in_rest'   => array(
						'schema' => array(
							'items' => array(
								'type' => 'string',
							),
						),
					),
					'type'           => 'array',
					'default'        => [],
					'single'         => true,
				)
			);

			// Setting fields.
			register_setting(
				'boldblocks',
				'boldblocks_pattern_categories',
				[
					'type'         => 'array',
					'show_in_rest' => array(
						'schema' => array(
							'items' => array(
								'type'       => 'object',
								'properties' => array(
									'name'  => array(
										'type' => 'string',
									),
									'label' => array(
										'type' => 'string',
									),
								),
							),
						),
					),
					'default'      => [],
				]
			);

			register_meta(
				'post',
				'boldblocks_pattern_block_types',
				array(
					'object_subtype' => 'boldblocks_pattern',
					'type'           => 'array',
					'show_in_rest'   => array(
						'schema' => array(
							'type'  => 'array',
							'items' => array(
								'type' => 'string',
							),
						),
					),
					'default'        => [],
					'single'         => true,
				)
			);

			register_meta(
				'post',
				'boldblocks_pattern_disabled_inserter',
				array(
					'object_subtype' => 'boldblocks_pattern',
					'show_in_rest'   => true,
					'type'           => 'boolean',
					'default'        => false,
					'single'         => true,
				)
			);

			register_setting(
				'boldblocks',
				'boldblocks_pattern_categories_all_label',
				[
					'type'         => 'string',
					'show_in_rest' => true,
					'default'      => __( 'All patterns by Boldblocks', 'content-blocks-builder' ),
				]
			);
		}

		/**
		 * Enqueue editor assets
		 *
		 * @return void
		 */
		public function enqueue_block_editor_assets() {
			// Custom blocks access file.
			$patterns_asset = $this->the_plugin_instance->include_file( 'build/patterns.asset.php' );

			// Scripts.
			wp_enqueue_script(
				'boldblocks-patterns',
				$this->the_plugin_instance->get_file_uri( '/build/patterns.js' ),
				$patterns_asset['dependencies'] ?? [],
				$this->the_plugin_instance->get_script_version( $patterns_asset ),
				true
			);

			// Add translation.
			wp_set_script_translations( 'boldblocks-patterns', 'content-blocks-builder' );

			// Load all patterns for registration.
			wp_localize_script(
				'boldblocks-patterns',
				'BoldblocksPatterns',
				[ 'allowedBlocks' => $this->get_pattern_allowed_blocks() ]
			);

			// Load pattern library url.
			wp_localize_script(
				'boldblocks-custom-blocks',
				'BoldblocksPatternLibrary',
				[ 'URL' => $this->get_pattern_library_url() ]
			);
		}

		/**
		 * Get the list of blocks that allow creating patterns from toolbar menu.
		 */
		public function get_pattern_allowed_blocks() {
			return apply_filters(
				'boldblocks_get_pattern_allowed_blocks',
				[
					'core/group',
					'core/cover',
					'core/columns',
					'core/media-text',
				]
			);
		}

		/**
		 * Register all custom patterns
		 *
		 * @return void
		 */
		public function register_patterns() {
			// Register custom categories.
			$pattern_categories = get_option( 'boldblocks_pattern_categories', [] );
			if ( is_array( $pattern_categories ) ) {
				foreach ( $pattern_categories as $category ) {
					if ( ! empty( $category['name'] ) && ! empty( $category['label'] ) ) {
						if ( ! \WP_Block_Pattern_Categories_Registry::get_instance()->is_registered( $category['name'] ) ) {
							register_block_pattern_category( $category['name'], [ 'label' => $category['label'] ] );
						}
					}
				}
			}

			// Register a custom category for patterns made by BoldBlocks.
			register_block_pattern_category( 'boldblocks', [ 'label' => get_option( 'boldblocks_pattern_categories_all_label', _x( 'All patterns by BoldBlocks', 'Block pattern category', 'content-blocks-builder' ) ) ] );

			if ( apply_filters( 'boldblocks_load_patterns', true ) ) {
				// Query all patterns.
				$pattern_posts = $this->get_all_custom_patterns();

				foreach ( $pattern_posts as $pattern_name => $pattern_settings ) {
					register_block_pattern(
						sprintf( 'boldblocks/%s', $pattern_name ),
						$pattern_settings
					);
				}
			}
		}

		/**
		 * Get all custom patterns.
		 *
		 * @return array
		 */
		public function get_all_custom_patterns() {
			if ( ! $this->all_custom_patterns ) {
				$this->all_custom_patterns = $this->get_posts();
			}

			return $this->all_custom_patterns;
		}

		/**
		 * Query posts and cache the results.
		 *
		 * @param bool $force_refresh Optional. Whether to force the cache to be refreshed. Default false.
		 * @return array Array of WP_Post objects.
		 */
		public function get_posts( $force_refresh = false ) {
			$cache_key = 'bb_pattern_posts';

			$posts = get_transient( $cache_key );

			// If nothing is found, build the object.
			if ( true === $force_refresh || false === $posts ) {
				// Query posts.
				$posts = $this->query_posts();

				if ( ! is_wp_error( $posts ) && count( $posts ) > 0 ) {
					set_transient( $cache_key, $posts, HOUR_IN_SECONDS );
				}
			}

			return $posts;
		}

		/**
		 * Query and parse patterns
		 *
		 * @return array
		 */
		public function query_posts() {
			// Query all patterns.
			$raw_posts = get_posts(
				[
					'post_type'      => 'boldblocks_pattern',
					'posts_per_page' => $this->max_items,
					'orderby'        => [
						'menu_order' => 'DESC',
						'date'       => 'DESC',
					],
				]
			);

			$pattern_posts = [];

			foreach ( $raw_posts as $pattern_post ) {
				$blocks = parse_blocks( $pattern_post->post_content );
				if ( count( $blocks ) === 0 ) {
					continue;
				}

				$categories = get_post_meta( $pattern_post->ID, 'boldblocks_pattern_categories', true );
				if ( ! is_array( $categories ) ) {
					$categories = [];
				} else {
					$categories = array_map(
						function( $item ) {
							return $item['name'] ?? '';
						},
						$categories
					);
				}
				$categories[] = 'boldblocks';

				$pattern_settings = [
					'title'       => wp_strip_all_tags( $pattern_post->post_title ),
					'content'     => $pattern_post->post_content,
					'categories'  => $categories,
					'description' => get_post_meta( $pattern_post->ID, 'boldblocks_pattern_description', true ) ?? '',
					'inserter'    => ! ( get_post_meta( $pattern_post->ID, 'boldblocks_pattern_disabled_inserter', true ) ?? false ),
				];

				$keywords = get_the_terms( $pattern_post, 'boldblocks_pattern_keywords' );
				if ( $keywords && ! is_wp_error( $keywords ) ) {
					$keywords = wp_list_pluck( $keywords, 'name' );
				} else {
					$keywords = false;
				}

				if ( $keywords ) {
					$pattern_settings['keywords'] = $keywords;
				}

				// Add blockTypes.
				$pattern_settings['blockTypes'] = [];
				if ( in_array( 'header', $categories, true ) ) {
					$pattern_settings['blockTypes'][] = 'core/template-part/header';
				}

				if ( in_array( 'footer', $categories, true ) ) {
					$pattern_settings['blockTypes'][] = 'core/template-part/footer';
				}

				if ( in_array( 'query', $categories, true ) ) {
					$pattern_settings['blockTypes'][] = 'core/query';
				}

				$pattern_posts[ sanitize_key( $pattern_post->post_name ) ] = $pattern_settings;

			}

			return $pattern_posts;
		}

		/**
		 * Clear transient cache
		 *
		 * @return void
		 */
		public function clear_transient_cache() {
			delete_transient( 'bb_pattern_posts' );
		}

		/**
		 * Add custom columns to custom post type
		 *
		 * @param  array $columns The array of columns.
		 * @return array
		 */
		public function add_pattern_custom_columns( $columns ) {
			$custom_columns = array(
				'pattern_name' => esc_html__( 'Pattern name', 'content-blocks-builder' ),
			);

			// Add after title column.
			$return = array_slice( $columns, 0, 2, true ) + $custom_columns + array_slice( $columns, 2, count( $columns ) - 2, true );

			return $return;
		}

		/**
		 * Manage custom columns.
		 *
		 * @param string $column  the column name.
		 * @param int    $post_id the post id.
		 */
		public function manage_pattern_columns( $column, $post_id ) {
			switch ( $column ) {
				case 'pattern_name':
					echo esc_html( sprintf( _x( 'boldblocks/%s', 'Pattern name' ), sanitize_key( get_post_field( 'post_name', $post_id ) ) ) );
					break;

				// Just break out of the switch statement for everything else.
				default:
					break;
			}
		}

		/**
		 * Add admin menu for patterns: Add New Pattern, Pattern Categories
		 *
		 * @return void
		 */
		public function add_admin_menu() {
			// Make "Custom Blocks" as parent slug.
			$parent_slug = 'edit.php?post_type=boldblocks_block';

			// Add New Pattern.
			add_submenu_page( $parent_slug, __( 'Add New Pattern', 'content-blocks-builder' ), __( 'Add New Pattern', 'content-blocks-builder' ), 'publish_posts', 'post-new.php?post_type=boldblocks_pattern' );
		}

		/**
		 * Register an endpoint to query patterns.
		 *
		 * @return array
		 */
		public function register_get_patterns_endpoint() {
			register_rest_route(
				'boldblocks/v1',
				'/getPatterns/',
				array(
					'methods'             => 'GET',
					'callback'            => [ $this, 'get_patterns' ],
					'permission_callback' => '__return_true',
				)
			);

			register_rest_route(
				'boldblocks/v1',
				'/getFullPatternData/',
				array(
					'methods'             => 'GET',
					'callback'            => [ $this, 'get_full_pattern_data' ],
					'permission_callback' => '__return_true',
				)
			);
		}

		/**
		 * Query patterns.
		 *
		 * @param WP_Rest_Request $request
		 * @return WP_Rest_Response
		 */
		public function get_patterns( $request ) {
			// Get all patterns.
			$data = $this->get_all_patterns();

			if ( is_array( $data ) ) {
				return wp_send_json( array_values( $data ) );
			}

			return wp_send_json( $data );
		}

		/**
		 * Build patterns data
		 *
		 * @param boolean $force_refresh
		 * @return array
		 */
		private function get_all_patterns( $force_refresh = false ) {
			$cache_key = $this->get_patterns_cache_key();
			$data      = get_transient( $cache_key );

			if ( $force_refresh || false === $data ) {
				$response = wp_remote_get(
					$this->pattern_library_url . '/wp-json/wp/v2/boldblocks-patterns',
					[
						'timeout'   => 120,
						'sslverify' => false,
						'body'      => [
							'_load_all'   => 1,
							'_fields'     =>
							'id,slug,meta,menu_order,order,title,description,keywords,thumbnail,boldblocks_pattern_keywords,keywordIds,is_pro,has_pro_features',
							'api_version' => 2,
						],
					]
				);

				if ( ! is_wp_error( $response ) ) {
					$data = json_decode( wp_remote_retrieve_body( $response ), true );

					// Re-index the data.
					$data = array_column( $data, null, 'id' );

					// Update cache.
					$this->update_patterns_cache( $data, $cache_key );
				}
			}

			return $data;
		}

		/**
		 * Define patterns cache key
		 *
		 * @return string
		 */
		private function get_patterns_cache_key() {
			return 'bb_patterns_store';
		}

		/**
		 * Update patterns cache.
		 */
		private function update_patterns_cache( $patterns, $cache_key, $expired_time = DAY_IN_SECONDS ) {
			set_transient( $cache_key, $patterns, $expired_time );
		}

		/**
		 * Get full data for patterns
		 *
		 * @param WP_Rest_Request $request
		 * @return array
		 */
		public function get_full_pattern_data( $request ) {
			// Get all patterns.
			$all_patterns = $this->get_all_patterns();

			$pattern_ids = $request->get_param( 'patternIds' );

			$response = wp_remote_get(
				$this->pattern_library_url . '/wp-json/wp/v2/boldblocks-patterns',
				[
					'timeout'   => 120,
					'sslverify' => false,
					'body'      => [
						'_fields'      =>
						'id,slug,meta,menu_order,order,title,description,keywords,thumbnail,boldblocks_pattern_keywords,keywordIds,is_pro,has_pro_features',
						'api_version'  => 2,
						'include_data' => true,
						'include'      => $pattern_ids,
					],
				]
			);

			if ( ! is_wp_error( $response ) ) {
				$data = json_decode( wp_remote_retrieve_body( $response ), true );

				// Update cache.
				foreach ( $data as $post ) {
					$all_patterns[ $post['id'] ] = $post;
				}

				// Update cache.
				$this->update_patterns_cache( $all_patterns, $this->get_patterns_cache_key() );
			}

			return $data;
		}

		/**
		 * Register an endpoint to query pattern keywords.
		 *
		 * @return array
		 */
		public function register_get_pattern_keywords_endpoint() {
			register_rest_route(
				'boldblocks/v1',
				'/getPatternKeywords/',
				array(
					'methods'             => 'GET',
					'callback'            => [ $this, 'get_pattern_keywords' ],
					'permission_callback' => '__return_true',
				)
			);
		}

		/**
		 * Query pattern keywords.
		 *
		 * @param WP_Rest_Request $request
		 * @return WP_Rest_Response
		 */
		public function get_pattern_keywords( $request ) {
			$cache_key = 'bb_pattern_keywords_store';
			$data      = get_transient( $cache_key );
			if ( false === $data ) {
				$response = wp_remote_get(
					$this->pattern_library_url . '/wp-json/wp/v2/boldblocks_pattern_keywords',
					[
						'timeout'   => 120,
						'sslverify' => false,
						'body'      => [
							'per_page'   => 100,
							'orderby'    => 'order',
							'order'      => 'desc',
							'hide_empty' => 1,
							'_fields'    => 'id,count,name,slug,is_design_field',
						],
					]
				);

				if ( ! is_wp_error( $response ) ) {
					$data = json_decode( wp_remote_retrieve_body( $response ), true );

					set_transient( $cache_key, $data, 7 * DAY_IN_SECONDS );
				}
			}

			return wp_send_json( $data );
		}

		/**
		 * Register an endpoint to query pattern categories.
		 *
		 * @return array
		 */
		public function register_get_pattern_categories_endpoint() {
			register_rest_route(
				'boldblocks/v1',
				'/getPatternCategories/',
				array(
					'methods'             => 'GET',
					'callback'            => [ $this, 'get_pattern_categories' ],
					'permission_callback' => '__return_true',
				)
			);
		}

		/**
		 * Build pattern categories.
		 *
		 * @param WP_Rest_Request $request
		 * @return WP_Rest_Response
		 */
		public function get_pattern_categories( $request ) {
			$categories = \WP_Block_Pattern_Categories_Registry::get_instance()->get_all_registered();

			return $categories;
		}

		/**
		 * Add custom block's meta fields to meta revisioning.
		 *
		 * @param array $fields
		 * @return array
		 */
		public function add_fields_to_revision_meta_keys( $fields ) {
			return array_merge( $fields, [ 'boldblocks_pattern_categories' ] );
		}

		/**
		 * Register custom rest fields for easier getting data.
		 *
		 * @return void
		 */
		public function register_rest_fields() {
			register_rest_field(
				'boldblocks_pattern',
				'keywords',
				array(
					'get_callback'    => function( $params ) {
						$keywords = wp_list_pluck( wp_get_object_terms( $params['id'], 'boldblocks_pattern_keywords' ), 'name' );

						return \implode( ',', $keywords );
					},
					'update_callback' => function( $value, $post ) {
						wp_set_post_terms( $post->ID, $value, 'boldblocks_pattern_keywords' );
					},
					'schema'          => array(
						'type' => 'string',
					),
				)
			);
		}

		/**
		 * Placeholder text. Default 'Add title'.
		 *
		 * @param string $text
		 * @return string
		 */
		public function enter_title_here( $text ) {
			$post_type = get_post_type();
			if ( 'boldblocks_pattern' === $post_type ) {
				$text = __( 'Add pattern title', 'content-blocks-builder' );
			}

			return $text;
		}

		/**
		 * Get the pattern library url
		 *
		 * @return string
		 */
		public function get_pattern_library_url() {
			return $this->pattern_library_url;
		}
	}
endif;
