<?php

/**
 * The settings: import/export blocks, variants, patterns.
 *
 * @package   BoldBlocks
 * @author    Phi Phan <mrphipv@gmail.com>
 * @copyright Copyright (c) 2022, Phi Phan
 */
namespace BoldBlocks;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;
if ( !class_exists( 'Settings' ) ) {
    /**
     * The controller class for the settings.
     */
    class Settings extends CoreComponent
    {
        /**
         * Setting page's hook suffix.
         *
         * @var string
         */
        protected  $hook_suffix ;
        /**
         * Store the version of core blocks
         *
         * @var string
         */
        protected  $core_blocks_version = '2.2.9' ;
        /**
         * Run main hooks
         *
         * @return void
         */
        public function run()
        {
            // Add admin toolbar.
            add_action( 'in_admin_header', [ $this, 'in_admin_header' ] );
            // Create the settings page.
            add_action( 'admin_menu', [ $this, 'add_admin_page' ] );
            // Enqueue settings script.
            add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_settings_scripts' ] );
            // Do setting up stuff when the plugin is activated.
            add_action( 'content_block_builder_activate', [ $this, 'run_the_plugin_setup' ] );
            // Add the settings page link to plugin list screen.
            add_filter( 'plugin_action_links_' . plugin_basename( BOLDBLOCKS_CBB_ROOT_FILE ), [ $this, 'plugin_settings_links' ] );
            // Maybe update core blocks.
            add_action( 'init', [ $this, 'maybe_update_core_blocks' ] );
            // Redirect to the getting started page.
            add_action( 'admin_init', [ $this, 'boldblocks_activation_redirect' ] );
            // Add rest api endpoint to query docs.
            add_action( 'rest_api_init', [ $this, 'register_docs_endpoint' ] );
        }
        
        /**
         * Print the admin toolbar.
         *
         * @return void
         */
        public function in_admin_header()
        {
            $screen = get_current_screen();
            
            if ( 'boldblocks_block_page_cbb-settings' === $screen->base ) {
                ?>
				<div class="cbb-settings-header">
					<h1><?php 
                printf( __( '<strong>Content Blocks Builder</strong>', 'content-blocks-builder' ) );
                ?> <code><?php 
                echo  esc_html( $this->the_plugin_instance->get_plugin_version() ) ;
                ?></code></h1>
					<ul>
						<li><a href="https://wordpress.org/support/plugin/content-blocks-builder/" target="_blank"><span class="dashicons dashicons-editor-help"></span> <?php 
                printf( __( 'Help & Support', 'content-blocks-builder' ) );
                ?></a></li>
						<li><a href="https://wordpress.org/support/plugin/content-blocks-builder/reviews/#new-post" target="_blank"><span class="dashicons dashicons-star-filled"></span> <?php 
                printf( __( 'Review', 'content-blocks-builder' ) );
                ?></a></li>
						<?php 
                if ( cbb_fs()->is_not_paying() ) {
                    printf( '<li class="go-premium"><a href="https://contentblocksbuilder.com/pro/?utm_source=CBB+Free&utm_campagin=CBB+Upgrade&utm_medium=link&utm_content=header" target="_blank"><span class="dashicons dashicons-superhero-alt"></span> %s</a></li>', __( 'Go Premium', 'content-blocks-builder' ) );
                }
                ?>
					</ul>
				</div>
				<?php 
            }
        
        }
        
        /**
         * Create the admin page
         *
         * @return void
         */
        public function add_admin_page()
        {
            // Make "Custom Blocks" as parent slug.
            $parent_slug = 'edit.php?post_type=boldblocks_block';
            $this->hook_suffix = add_submenu_page(
                $parent_slug,
                __( 'Content Blocks Builder' ),
                __( 'Settings', 'content-blocks-builder' ),
                'manage_options',
                'cbb-settings',
                function () {
                ?>
					<div class="wrap boldblocks-settings js-boldblocks-settings-root">
					</div>
					<?php 
            }
            );
        }
        
        /**
         * Enqueue scripts for the settings page.
         *
         * @param string $hook_suffix
         * @return void
         */
        public function enqueue_settings_scripts( $hook_suffix )
        {
            // Only load scripts for the settings page.
            
            if ( 'boldblocks_block_page_cbb-settings' === $hook_suffix ) {
                // Settings asset file.
                $settings_asset = $this->the_plugin_instance->include_file( "build/settings{$this->the_plugin_instance->get_script_suffix()}.asset.php" );
                // Enqueue scripts.
                wp_enqueue_script(
                    'boldblocks-settings',
                    $this->the_plugin_instance->get_file_uri( "build/settings{$this->the_plugin_instance->get_script_suffix()}.js" ),
                    $settings_asset['dependencies'] ?? [],
                    $this->the_plugin_instance->get_script_version( $settings_asset ),
                    true
                );
                // Enqueue style.
                wp_enqueue_style(
                    'boldblocks-settings',
                    $this->the_plugin_instance->get_file_uri( 'build/settings.css' ),
                    [],
                    $this->the_plugin_instance->get_script_version( $settings_asset )
                );
                // Load components style.
                wp_enqueue_style( 'wp-components' );
            }
        
        }
        
        /**
         * Run activated stuff
         * https://wearnhardt.com/2020/03/redirecting-after-plugin-activation/
         *
         * @return void
         */
        public function run_the_plugin_setup()
        {
            // Deploy core blocks.
            
            if ( is_admin() && !get_option( 'boldblocks_deployed_core_blocks' ) ) {
                // Update core blocks.
                $this->update_core_blocks();
                // Add the marker to the settings to make sure this logic only runs one time.
                add_option( 'boldblocks_deployed_core_blocks', 1 );
            }
            
            // Redirect to the getting started page, ignore bulk activation.
            if ( !(isset( $_REQUEST['action'] ) && 'activate-selected' === $_REQUEST['action'] && (isset( $_POST['checked'] ) && count( $_POST['checked'] ) > 1)) ) {
                add_option( 'boldblocks_activation_redirect', wp_get_current_user()->ID );
            }
        }
        
        /**
         * Redirect to the getting started page.
         *
         * @return void
         */
        public function boldblocks_activation_redirect()
        {
            if ( !cbb_fs()->is_activation_mode() ) {
                // Make sure it's the correct user.
                
                if ( !wp_doing_ajax() && intval( get_option( 'boldblocks_activation_redirect', false ) ) === wp_get_current_user()->ID ) {
                    // Make sure we don't redirect again after this one.
                    delete_option( 'boldblocks_activation_redirect' );
                    wp_safe_redirect( admin_url( '/edit.php?post_type=boldblocks_block&page=cbb-settings&tab=getting-started' ) );
                    exit;
                }
            
            }
        }
        
        /**
         * Add the settings page link to the plugin admin screen.
         *
         * @param array $links
         * @return array
         */
        public function plugin_settings_links( $links ) : array
        {
            $label = esc_html__( 'Settings', 'content-blocks-builder' );
            $slug = 'cbb-settings';
            array_unshift( $links, "<a href='edit.php?post_type=boldblocks_block&page={$slug}'>{$label}</a>" );
            if ( cbb_fs()->is_not_paying() ) {
                $links[] = sprintf( '<a href="%1$s" target="_blank" style="color: #d20962">%2$s</a>', 'https://contentblocksbuilder.com/pro/?utm_source=CBB+Free&utm_campagin=CBB+Upgrade&utm_medium=link&utm_content=action-link', __( 'Go Premium', 'content-blocks-builder' ) );
            }
            return $links;
        }
        
        /**
         * Maybe update core blocks
         *
         * @return void
         */
        public function maybe_update_core_blocks()
        {
            
            if ( is_admin() && current_user_can( 'manage_options' ) ) {
                $core_blocks_version = get_option( 'boldblocks_core_blocks_version' );
                if ( !$core_blocks_version || $core_blocks_version !== $this->core_blocks_version ) {
                    $this->update_core_blocks();
                }
            }
        
        }
        
        /**
         * Parse core blocks
         *
         * @return array
         */
        private function get_core_blocks()
        {
            $core_blocks = false;
            $core_blocks_file_path = $this->the_plugin_instance->get_file_path( 'data/core-blocks/blocks.json' );
            
            if ( \file_exists( $core_blocks_file_path ) ) {
                $core_blocks = \file_get_contents( $core_blocks_file_path );
                $core_blocks = \json_decode( $core_blocks, true );
            }
            
            return $core_blocks;
        }
        
        /**
         * Update core blocks
         *
         * @return void
         */
        private function update_core_blocks()
        {
            
            if ( is_admin() ) {
                $core_blocks_data = $this->get_core_blocks();
                
                if ( $core_blocks_data ) {
                    $core_blocks = $core_blocks_data['blocks'] ?? [];
                    foreach ( $core_blocks as $block ) {
                        $name = $block['slug'] ?? '';
                        $found_posts = get_posts( [
                            'post_type'      => 'boldblocks_block',
                            'name'           => $name,
                            'posts_per_page' => 1,
                        ] );
                        // There is no block with that slug.
                        
                        if ( !(is_array( $found_posts ) && count( $found_posts ) > 0) ) {
                            $title = $block['title'] ?? '';
                            $content = $block['content'] ?? '';
                            $meta = $block['meta'] ?? [];
                            // Insert new block.
                            $block_id = wp_insert_post( [
                                'post_type'    => 'boldblocks_block',
                                'post_title'   => $title,
                                'post_content' => $content,
                                'post_name'    => $name,
                                'post_status'  => 'publish',
                                'meta_input'   => $meta,
                            ] );
                        } else {
                            $found_post = $found_posts[0];
                            $meta = $block['meta'] ?? [];
                            $updating_meta = [];
                            $title = $found_post->post_title ?? '';
                            $description = $meta['boldblocks_block_description'] ?? '';
                            if ( is_array( $description ) && count( $description ) > 0 ) {
                                $description = $description[0] ?? '';
                            }
                            $updating_meta['boldblocks_block_description'] = $description;
                            $icon = $meta['boldblocks_block_icon'] ?? '';
                            if ( is_array( $icon ) && count( $icon ) > 0 ) {
                                $icon = $icon[0] ?? '';
                            }
                            $updating_meta['boldblocks_block_icon'] = $icon;
                            $supports = $meta['boldblocks_block_supports'] ?? [];
                            if ( !empty($supports) ) {
                                $updating_meta['boldblocks_block_supports'] = $supports;
                            }
                            $enable_repeater = $meta['boldblocks_enable_repeater'] ?? '';
                            if ( is_array( $enable_repeater ) && count( $enable_repeater ) > 0 ) {
                                $enable_repeater = $enable_repeater[0] ?? '';
                            }
                            
                            if ( $enable_repeater ) {
                                $parent_title = $meta['boldblocks_parent_block_title'] ?? '';
                                if ( is_array( $parent_title ) && count( $parent_title ) > 0 ) {
                                    $parent_title = $parent_title[0] ?? '';
                                }
                                $updating_meta['boldblocks_parent_block_title'] = $parent_title;
                                $parent_description = $meta['boldblocks_parent_block_description'] ?? '';
                                if ( is_array( $parent_description ) && count( $parent_description ) > 0 ) {
                                    $parent_description = $parent_description[0] ?? '';
                                }
                                $updating_meta['boldblocks_parent_block_description'] = $parent_description;
                                $parent_icon = $meta['boldblocks_parent_block_icon'] ?? '';
                                if ( is_array( $parent_icon ) && count( $parent_icon ) > 0 ) {
                                    $parent_icon = $parent_icon[0] ?? '';
                                }
                                $updating_meta['boldblocks_parent_block_icon'] = $parent_icon;
                                $parent_supports = $meta['boldblocks_parent_block_supports'] ?? [];
                                if ( !empty($parent_supports) ) {
                                    $updating_meta['boldblocks_parent_block_supports'] = $parent_supports;
                                }
                            }
                            
                            $updating_data = [
                                'ID'         => $found_post->ID,
                                'post_title' => $block['title'] ?? '',
                                'meta_input' => $updating_meta,
                            ];
                            $post_id = wp_update_post( $updating_data );
                        }
                    
                    }
                    update_option( 'boldblocks_core_blocks_version', $this->core_blocks_version );
                }
            
            }
        
        }
        
        /**
         * Build a custom endpoint to query docs.
         *
         * @return void
         */
        public function register_docs_endpoint()
        {
            register_rest_route( 'boldblocks/v1', '/getDocs/', array(
                'methods'             => 'GET',
                'callback'            => [ $this, 'get_docs' ],
                'permission_callback' => function () {
                return current_user_can( 'publish_posts' );
            },
            ) );
        }
        
        /**
         * Get docs.
         *
         * @param WP_REST_Request $request The request object.
         * @return void
         */
        public function get_docs( $request )
        {
            $cache_key = 'bb_docs';
            $data = get_transient( $cache_key );
            $patterns_component = $this->the_plugin_instance->get_component( 'BoldBlocks\\Patterns' );
            $pattern_library_url = ( $patterns_component ? $patterns_component->get_pattern_library_url() : 'https://boldpatterns.net' );
            
            if ( false === $data ) {
                $response = wp_remote_get( $pattern_library_url . '/wp-json/api.boldblocks/v1/getDocs', [
                    'timeout'   => 120,
                    'sslverify' => false,
                ] );
                
                if ( !is_wp_error( $response ) ) {
                    $data = json_decode( wp_remote_retrieve_body( $response ), true );
                    if ( $data ) {
                        set_transient( $cache_key, $data, DAY_IN_SECONDS );
                    }
                }
            
            }
            
            wp_send_json( [
                'data'    => $data,
                'success' => true,
            ] );
        }
    
    }
}