<?php
/**
 * Handle custom styles for themes
 *
 * @package   BoldBlocks
 * @author    Phi Phan <mrphipv@gmail.com>
 * @copyright Copyright (c) 2022, Phi Phan
 */

namespace BoldBlocks;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'Theme' ) ) :
	/**
	 * The controller class for theme.
	 */
	class Theme extends CoreComponent {
		/**
		 * Supported themes.
		 *
		 * @var array
		 */
		protected $themes = [
			'blockbase'       => [
				'spacing_horizontal_padding' => 'var(--wp--custom--gap--horizontal)',
				'stylesheet'                 => 'blockbase',
			],
			'twentytwentytwo' => [
				'spacing_horizontal_padding' => 'var(--wp--custom--spacing--outer)',
				'stylesheet'                 => 'blockbase',
			],
		];

		/**
		 * Run main hooks
		 *
		 * @return void
		 */
		public function run() {
			// Enqueue styles for frontend.
			add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts_for_theme' ], 99999 );

			// Enqueue styles for the iframe editor.
			add_filter( 'block_editor_settings_all', [ $this, 'enqueue_style_for_the_editor' ] );

			// Add transparency to color palette.
			add_filter( 'wp_theme_json_data_default', [ $this, 'filter_theme_json_data' ] );
		}

		/**
		 * Enqueue custom styles for popular themes.
		 *
		 * @return void
		 */
		public function enqueue_scripts_for_theme() {
			$css_variables_stylesheet = '';

			// Get current theme.
			$theme_slug = get_stylesheet();

			$global_styles_handle = '';
			if ( isset( wp_styles()->registered['global-styles'] ) ) {
				$global_styles_handle = 'global-styles';
			} elseif ( isset( wp_styles()->registered['global-styles-css-custom-properties'] ) ) {
				$global_styles_handle = 'global-styles-css-custom-properties';
			}

			// Has global style.
			if ( $global_styles_handle ) {
				$css_variables_stylesheet = $this->get_custom_css_variables();

				// Enqueue css variables.
				if ( $css_variables_stylesheet ) {
					wp_add_inline_style( $global_styles_handle, $css_variables_stylesheet );
				}
			}

			if ( \in_array( $theme_slug, array_keys( $this->themes ), true ) || $css_variables_stylesheet ) {
				$stylesheet_file = _wp_array_get( $this->themes, [ $theme_slug, 'stylesheet' ], 'blockbase' );
				if ( \is_readable( $this->the_plugin_instance->get_file_path( "build/{$stylesheet_file}.asset.php" ) ) ) {
					// Asset file.
					$asset_file = $this->the_plugin_instance->include_file( "build/{$stylesheet_file}.asset.php" );

					// Enqueue style.
					wp_enqueue_style(
						"bb-{$theme_slug}",
						$this->the_plugin_instance->get_file_uri( "build/{$stylesheet_file}.css" ),
						[],
						$this->the_plugin_instance->get_script_version( $asset_file )
					);
				}
			}
		}

		/**
		 * Enqueue scripts/styles for the editor
		 *
		 * @param array $editor_settings
		 * @return void
		 */
		public function enqueue_style_for_the_editor( $editor_settings ) {
			// Get css variables.
			$css_variables_stylesheet = $this->get_custom_css_variables();

			if ( $css_variables_stylesheet ) {
				$editor_settings['styles'][] = [ 'css' => $css_variables_stylesheet ];
			}

			// Get current theme.
			$theme_slug = get_stylesheet();
			if ( \in_array( $theme_slug, array_keys( $this->themes ), true ) || $css_variables_stylesheet ) {
				$stylesheet_file = _wp_array_get( $this->themes, [ $theme_slug, 'stylesheet' ], 'blockbase' );
				if ( \is_readable( $this->the_plugin_instance->get_file_path( "build/{$stylesheet_file}.css" ) ) ) {
					// Get css content.
					$css = \file_get_contents( $this->the_plugin_instance->get_file_path( "build/{$stylesheet_file}.css" ) );
					if ( $css ) {
						$editor_settings['styles'][] = [ 'css' => $css ];
					}
				}
			}

			return $editor_settings;
		}

		/**
		 * Enqueue custom css variables.
		 *
		 * @return void
		 */
		public function get_custom_css_variables() {
			$stylesheet = '';

			// Get current theme.
			$theme_slug = get_stylesheet();

			// Get theme settings.
			$theme_settings = \WP_Theme_JSON_Resolver::get_merged_data()->get_settings();

			$spacing_baseline = _wp_array_get( $theme_settings, [ 'custom', 'boldblocks', 'spacing', 'baseline' ], '' );
			if ( empty( $spacing_baseline ) ) {
				if ( \in_array( $theme_slug, array_keys( $this->themes ), true ) ) {
					$spacing_baseline = _wp_array_get( $this->themes, [ $theme_slug, 'spacing_horizontal_padding' ], '' );
				} else {
					// Most of block based themes using spacing->outer or gap->horizontal for the main horizontal padding.
					$theme_spacing_baseline = _wp_array_get( $theme_settings, [ 'custom', 'spacing', 'outer' ] );
					if ( $theme_spacing_baseline ) {
						$spacing_baseline = 'var(--wp--custom--spacing--outer)';
					} else {
						$theme_spacing_baseline = _wp_array_get( $theme_settings, [ 'custom', 'gap', 'horizontal' ] );
						if ( $theme_spacing_baseline ) {
							$spacing_baseline = 'var(--wp--custom--gap--horizontal)';
						}
					}
				}

				if ( $spacing_baseline ) {
					$stylesheet = 'body{--wp--custom--boldblocks--spacing--baseline:' . $spacing_baseline . '}';
				} else {
					$stylesheet = 'body{--wp--custom--boldblocks--spacing--baseline: 1.25rem}';
				}
			}

			$theme_colors = _wp_array_get( $theme_settings, [ 'color', 'palette' ] );
			if ( ! array_search( 'transparent', array_column( $theme_colors, 'color' ), true ) ) {
				$stylesheet .= '.has-transparent-color { color: transparent !important; } .has-transparent-background-color { background-color: transparent !important; } .has-transparent-border-color { border-color: transparent !important; }';
			}

			return $stylesheet;
		}

		/**
		 * Add transparency to the default color palette
		 *
		 * @param array $theme_json
		 * @return array
		 */
		public function filter_theme_json_data( $theme_json ) {
			$theme_json_data = $theme_json->get_data();
			if ( isset( $theme_json_data['settings'] ) && isset( $theme_json_data['settings']['color'] ) && isset( $theme_json_data['settings']['color']['palette'] ) ) {
				$color_palette = $theme_json_data['settings']['color']['palette']['default'] ?? [];
				if ( ! array_search( 'transparent', array_column( $color_palette, 'color' ), true ) ) {
					$color_palette[] = [
						'slug'  => 'transparent',
						'color' => 'transparent',
						'name'  => 'Transparent',
					];

					$updating_data = array(
						'version'  => 2,
						'settings' => array(
							'color' => array(
								'palette' => $color_palette,
							),
						),
					);
				}

				return $theme_json->update_with( $updating_data );
			}

			return $theme_json;
		}
	}
endif;
